#include "mastertab.h"
#include <dirent.h>
#include <vector>
#include <iostream>
#include <fstream>
#include <QString>

using namespace std;

void init_probas(QDoubleSpinBox *dsb){
    dsb->setMaximum(1.0);
    dsb->setMinimum(0.0);
    dsb->setSingleStep(0.05);
}

void select_choice(QComboBox *qcb, QString value){
    int i;
    for(i=0;i<qcb->count(); i++){
        if(qcb->itemText(i) == value) qcb->setCurrentIndex(i);
    }
}


MasterTab::MasterTab(){

    layout = new QVBoxLayout;

    rom_path = new LabelledWidget<QComboBox>("Rom");
    
    DIR *rom_dir = opendir("../ale/roms");
    if(rom_dir == NULL){
        std::cout << "Could not open ALE rom directory\n";
    }else{
        struct dirent *ent;
        while((ent = readdir(rom_dir))!= NULL){
            if(ent->d_name[0]=='.') continue;
            rom_path->getChild()->addItem(ent->d_name);
        }
    }

    colour_palette = new LabelledWidget<QComboBox>("Color Palette");
    colour_palette->getChild()->addItems({"SECAM", "PAL", "standard"});

    display_screen = new LabelledWidget<QCheckBox>("Display Screen");
    enable_sound = new LabelledWidget<QCheckBox>("Enable Sound");
    fresh_pop = new LabelledWidget<QCheckBox>("Fresh Pop");

    repeat_action_prob = new LabelledWidget<QDoubleSpinBox>("Repeat action proba");
    mut_rate = new LabelledWidget<QDoubleSpinBox>("Mutation rate");
    cross_rate = new LabelledWidget<QDoubleSpinBox>("Crossover rate");

    n_eval_p = new LabelledWidget<QSpinBox>("Nb tests / agent");

    n_eval_s = new LabelledWidget<QSpinBox>("Nb tests max by a slave on an agent");
    tourn_size = new LabelledWidget<QSpinBox>("Tournament Size");
    pop_size = new LabelledWidget<QSpinBox>("Population size");
    nb_iter = new LabelledWidget<QSpinBox>("Nb Iterations");

    init_probas(repeat_action_prob->getChild());
    init_probas(mut_rate->getChild());
    init_probas(cross_rate->getChild());

    this->loadCfg();


    layout->addWidget(rom_path->getWidget());

    layout->addWidget(display_screen->getWidget());
    layout->addWidget(enable_sound->getWidget());
    layout->addWidget(repeat_action_prob->getWidget());
    layout->addWidget(colour_palette->getWidget());
    layout->addWidget(fresh_pop->getWidget());
    layout->addWidget(n_eval_p->getWidget());
    layout->addWidget(n_eval_s->getWidget());
    layout->addWidget(mut_rate->getWidget());
    layout->addWidget(cross_rate->getWidget());
    layout->addWidget(tourn_size->getWidget());
    layout->addWidget(pop_size->getWidget());
    layout->addWidget(nb_iter->getWidget());

    this->setLayout(layout);

}


void MasterTab::loadCfg(){

    ifstream config;
    config.open("../src/config.config");

    if(config.is_open()){

        string line;
        while (getline(config,line)){

            if(line.length() < 2) continue;
            if(line.at(0) == '/' && line.at(1) == '/') continue;
            
            size_t pos = line.find("=");
            string param = line.substr(0,pos);
            string value = line.substr(pos+1);

            if(param == "romPath"){
                select_choice(rom_path->getChild(),
                    QString::fromStdString(
                        value.substr(value.find_last_of("/") + 1)
                    ));
	    	}else if(param == "displayScreen"){
                if(value == "true") display_screen->getChild()->setChecked(true);
    		}else if(param == "enableSound"){
    			if(value == "true") enable_sound->getChild()->setChecked(true);
    		}else if(param == "repeatActionProb"){
                repeat_action_prob->getChild()->setValue(
                    stod(value)
                );
    		}else if(param == "colourPalette"){
    			select_choice(
                    colour_palette->getChild(),
                    QString::fromStdString(value)
                );
            }else if(param == "freshPop"){
                if(value == "true") fresh_pop->getChild()->setChecked(true);
            }else if(param == "nEvalP"){
                n_eval_p->getChild()->setValue(
                    stoi(value)
                );
            }else if(param == "nEvalS"){
                n_eval_s->getChild()->setValue(
                    stoi(value)
                );
            }else if(param == "mutRate"){
                mut_rate->getChild()->setValue(
                    stod(value)
                );
            }else if(param == "crossRate"){
                cross_rate->getChild()->setValue(
                    stod(value)
                );
            }else if(param == "tournSize"){
                tourn_size->getChild()->setValue(
                    stoi(value)
                );
            }else if(param == "popSize"){
                pop_size->getChild()->setValue(
                    stoi(value)
                );
            }else if(param == "nbIter"){
                nb_iter->getChild()->setValue(
                    stoi(value)
                );
    		}

        }


    }else{
        cout << "Unable to find config.config file" << endl;
    }
}